==> ejc-budgetmanage-impl-3.0.0-20220328.024439-27-sources.jar: com/ejianc/business/change/controller/ChangedesignestimateController.java <==
package com.ejianc.business.change.controller;

import java.io.Serializable;

import com.ejianc.business.bedget.bean.DesignestimateEntity;
import com.ejianc.business.bedget.bean.DesignestimatedetailEntity;
import com.ejianc.business.bedget.service.IDesignestimateService;
import com.ejianc.business.bedget.vo.DesignestimatedetailVO;
import com.ejianc.business.change.bean.ChangedesignestimateEntity;
import com.ejianc.business.change.service.IChangedesignestimateService;
import com.ejianc.business.change.vo.ChangedesignestimateVO;
import com.ejianc.business.change.vo.ChangedesignestimatedetailVO;
import com.ejianc.foundation.support.api.IBillCodeApi;
import com.ejianc.framework.core.exception.BusinessException;
import com.ejianc.framework.core.util.ExcelExport;
import com.ejianc.support.idworker.util.IdWorker;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;
import com.alibaba.fastjson.JSONObject;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.*;
import javax.servlet.http.HttpServletResponse;

import com.ejianc.foundation.orgcenter.api.IOrgApi;
import com.ejianc.framework.core.response.CommonResponse;
import org.springframework.beans.factory.annotation.Autowired;
import com.ejianc.framework.core.context.InvocationInfoProxy;
import com.ejianc.framework.core.kit.mapper.BeanMapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.ejianc.framework.core.response.Parameter;
import com.ejianc.framework.core.response.QueryParam;



/**
 * 设计概算变更
 *
 * @author generator
 *
 */
@Controller
@RequestMapping("changedesignestimate")
public class ChangedesignestimateController implements Serializable {
	private static final long serialVersionUID = 1L;

    private Logger logger = LoggerFactory.getLogger(this.getClass());

    @Autowired
    private IBillCodeApi billCodeApi;
    @Autowired
    private IOrgApi iOrgApi;

    private static final String BILL_CODE = "change-budgetmanage-designestimate";//此处需要根据实际修改

    @Autowired
    private IChangedesignestimateService service;

    @Autowired
    private IDesignestimateService  iDesignestimateService;
    /**
     * 初始化更详情页面
     *
     * @param id
     * @return
     */
    @GetMapping(value = "queryDetailByDesignestimateId")
    @ResponseBody
    public CommonResponse<ChangedesignestimateVO> queryDetailByDesignestimateId(@RequestParam(value = "designestimateId") Long designestimateId) {
        DesignestimateEntity quotaanalysisVO = iDesignestimateService.selectById(designestimateId);
        ChangedesignestimateVO changeVO = BeanMapper.map(quotaanalysisVO, ChangedesignestimateVO.class);
        Long id = IdWorker.getId();
        changeVO.setDesignestimateId(designestimateId);
        changeVO.setBillState(null);
        changeVO.setCreateTime(null);
        changeVO.setCreateUserCode(null);
        changeVO.setTenantId(null);
        changeVO.setUpdateTime(null);
        changeVO.setUpdateUserCode(null);
        changeVO.setId(id);

        //设计概算清单
        List<DesignestimatedetailEntity> designestimateEntityList = quotaanalysisVO.getDesignestimatedetailEntities();
        if(designestimateEntityList!=null&&designestimateEntityList.size()>0){
            List<ChangedesignestimatedetailVO> changedesignestimatedetailVOS = new ArrayList<>();
            for(DesignestimatedetailEntity entity : designestimateEntityList){
                ChangedesignestimatedetailVO vo = BeanMapper.map(entity, ChangedesignestimatedetailVO.class);
                vo.setChangeMid(designestimateId);
                vo.setCreateTime(null);
                vo.setCreateUserCode(null);
                vo.setTenantId(null);
                vo.setUpdateTime(null);
                vo.setUpdateUserCode(null);
                vo.setRowState("edit");
                vo.setId(IdWorker.getId());
                changedesignestimatedetailVOS.add(vo);
            }

            //构建树形结构
            List<ChangedesignestimatedetailVO> rescChangedesignestimatedetailVOS = createTreeData(changedesignestimatedetailVOS);
            changeVO.setDesignestimatedetailEntities(rescChangedesignestimatedetailVOS);
        }

        return CommonResponse.success("查询详情成功！", changeVO);

    }

    /**
     * @Description saveOrUpdate 新增或者修改
     */
    @RequestMapping(value = "/saveChange", method = RequestMethod.POST)
    @ResponseBody
    public CommonResponse<ChangedesignestimateVO> saveChange(@RequestBody ChangedesignestimateVO saveOrUpdateVO) {
        return service.saveChange(saveOrUpdateVO);
    }

    /**
     * @Description saveOrUpdate 新增或者修改
     */
    @RequestMapping(value = "/saveOrUpdate", method = RequestMethod.POST)
    @ResponseBody
    public CommonResponse<ChangedesignestimateVO> saveOrUpdate(@RequestBody ChangedesignestimateVO saveorUpdateVO) {
    	ChangedesignestimateEntity entity = BeanMapper.map(saveorUpdateVO, ChangedesignestimateEntity.class);
    	if(entity.getId() == null || entity.getId() == 0){
            CommonResponse<String> billCode = billCodeApi.getCodeBatchByRuleCode(BILL_CODE, InvocationInfoProxy.getTenantid());
            if(billCode.isSuccess()) {
//                entity.setCode(billCode.getData());//此处需要根据实际修改 删除本行或者下一行
                entity.setBillCode(billCode.getData());//此处需要根据实际修改 删除本行或者上一行
            }else{
                throw new BusinessException("网络异常， 编码生成失败， 请稍后再试");
            }
        }
    	service.saveOrUpdate(entity, false);
    	ChangedesignestimateVO vo = BeanMapper.map(entity, ChangedesignestimateVO.class);
    	return CommonResponse.success("保存或修改单据成功！",vo);
    }

    /**
     * @Description queryDetail 查询详情
     * @param id
     */
    @RequestMapping(value = "/queryDetail", method = RequestMethod.GET)
    @ResponseBody
    public CommonResponse<ChangedesignestimateVO> queryDetail(Long id) {
    	ChangedesignestimateEntity entity = service.selectById(id);
    	ChangedesignestimateVO vo = BeanMapper.map(entity, ChangedesignestimateVO.class);
        return CommonResponse.success("查询详情数据成功！",vo);
    }

    /**
     * @Description delete 批量删除单据
     * @Param [ids]
     */
    @RequestMapping(value = "/delete", method = RequestMethod.POST)
    @ResponseBody
    public CommonResponse<String> delete(@RequestBody List<ChangedesignestimateVO> vos) {
        service.removeByIds(vos.stream().map(ChangedesignestimateVO::getId).collect(Collectors.toList()),true);
        return CommonResponse.success("删除成功！");
    }

    /**
     * @Description queryList 查询列表
     * @param param
     * @Return com.ejianc.framework.core.response.CommonResponse<java.lang.String>
     */
    @RequestMapping(value = "/queryList", method = RequestMethod.POST)
    @ResponseBody
    public CommonResponse<IPage<ChangedesignestimateVO>> queryList(@RequestBody QueryParam param) {
        /** 模糊搜索配置字段示例 */
        List<String> fuzzyFields = param.getFuzzyFields();
        /** 租户隔离 */
        //param.getParams().put("tenantId", new Parameter(QueryParam.EQ, InvocationInfoProxy.getTenantid()));

        /** 数据隔离 本下 没有组织orgId的删除下面代码 */
//        param.getParams().put("orgId",new Parameter(QueryParam.IN,iOrgApi.findChildrenByParentId(InvocationInfoProxy.getOrgId()).getData().stream().map(OrgVO::getId).collect(Collectors.toList())));

        IPage<ChangedesignestimateEntity> page = service.queryPage(param,false);
        IPage<ChangedesignestimateVO> pageData = new Page<>(page.getCurrent(), page.getSize(), page.getTotal());
  		pageData.setRecords(BeanMapper.mapList(page.getRecords(), ChangedesignestimateVO.class));

        return CommonResponse.success("查询列表数据成功！",pageData);
    }


    /**
     * @Description 导出
     * @param param
     * @Return void
     */
    @RequestMapping(value = "/excelExport", method = RequestMethod.POST)
    @ResponseBody
    public void excelExport(@RequestBody QueryParam param, HttpServletResponse response) {
        /** 模糊搜索配置字段示例 */
        List<String> fuzzyFields = param.getFuzzyFields();
        param.getParams().put("tenant_id",new Parameter(QueryParam.EQ,InvocationInfoProxy.getTenantid()));
        param.setPageIndex(1);
        param.setPageSize(-1);
        /** 数据隔离 本下 没有组织orgId的删除下面代码 */
//        param.getParams().put("orgId",new Parameter(QueryParam.IN,iOrgApi.findChildrenByParentId(InvocationInfoProxy.getOrgId()).getData().stream().map(OrgVO::getId).collect(Collectors.toList())));
        List<ChangedesignestimateEntity> list = service.queryList(param);
        //todo:字段翻译等等
        Map<String, Object> beans = new HashMap<>();
        beans.put("records", list);
        ExcelExport.getInstance().export("Changedesignestimate-export.xlsx", beans, response);
    }

    /**
     * @Description 参照
     * @param param
     * @Return void
     */
    @RequestMapping(value = "/refChangedesignestimateData", method = RequestMethod.GET)
    @ResponseBody
    public CommonResponse<IPage<ChangedesignestimateVO>> refChangedesignestimateData(@RequestParam Integer pageNumber, @RequestParam Integer pageSize,
                                                                        String condition,
                                                                        String searchObject,
                                                                        String searchText) {
        QueryParam param = new QueryParam();
        param.setPageSize(pageSize);
        param.setPageIndex(pageNumber);
        param.setSearchText(searchText);
        param.setSearchObject(searchObject);
        /** 租户隔离 */
        //param.getParams().put("tenantId", new Parameter(QueryParam.EQ, InvocationInfoProxy.getTenantid()));
        if(StringUtils.isNotEmpty(condition)){
            /** 处理condition */
            JSONObject _con = JSONObject.parseObject(condition);
        }

        IPage<ChangedesignestimateEntity> page = service.queryPage(param,false);
        IPage<ChangedesignestimateVO> pageData = new Page<>(page.getCurrent(), page.getSize(), page.getTotal());
        pageData.setRecords(BeanMapper.mapList(page.getRecords(), ChangedesignestimateVO.class));

        return CommonResponse.success("查询参照数据成功！",pageData);
     }

    public static List<ChangedesignestimatedetailVO> createTreeData(List<ChangedesignestimatedetailVO> list) {
        List<ChangedesignestimatedetailVO> resp = new ArrayList<>();
        List<String> rootItems = new ArrayList<String>();

        //循环list，放入listMap重
        Map<String, ChangedesignestimatedetailVO> listMap = new HashMap<>();
        for(ChangedesignestimatedetailVO item:list) {
            listMap.put(item.getTid().toString(), item);
        }

        for(int i =0; i<list.size(); i++) {
            ChangedesignestimatedetailVO item = list.get(i);
            String parentId = (item.getTpid() != null) ? item.getTpid().toString():"";
            ChangedesignestimatedetailVO parent = listMap.get(parentId);
            if(parent != null) {
                List<ChangedesignestimatedetailVO> child = (List<ChangedesignestimatedetailVO>) parent.getChildren();
                if(child != null) {
                    child.add(item);
                }else{
                    List<ChangedesignestimatedetailVO> children = new ArrayList<ChangedesignestimatedetailVO>();
                    children.add(item);
                    parent.setChildren(children);
                }
            } else {
                rootItems.add(item.getTid());
            }
        }

        for(String rootId : rootItems) {
            resp.add(listMap.get(rootId));
        }

        return resp;
    }
}

==> ejc-budgetmanage-impl-3.0.0-20220404.033202-49-sources.jar: com/ejianc/business/change/controller/ChangedesignestimateController.java <==
package com.ejianc.business.change.controller;

import java.io.Serializable;

import com.ejianc.business.bedget.bean.DesignestimateEntity;
import com.ejianc.business.bedget.bean.DesignestimatedetailEntity;
import com.ejianc.business.bedget.service.IDesignestimateService;
import com.ejianc.business.bedget.vo.DesignestimatedetailVO;
import com.ejianc.business.change.bean.ChangedesignestimateEntity;
import com.ejianc.business.change.service.IChangedesignestimateService;
import com.ejianc.business.change.vo.ChangedesignestimateVO;
import com.ejianc.business.change.vo.ChangedesignestimatedetailVO;
import com.ejianc.business.change.vo.ChangequotadesigndetailVO;
import com.ejianc.foundation.support.api.IBillCodeApi;
import com.ejianc.framework.core.exception.BusinessException;
import com.ejianc.framework.core.util.ExcelExport;
import com.ejianc.support.idworker.util.IdWorker;
import org.apache.commons.lang3.StringUtils;
import org.apache.http.util.TextUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.*;
import java.util.stream.Collectors;
import com.alibaba.fastjson.JSONObject;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.*;
import javax.servlet.http.HttpServletResponse;

import com.ejianc.foundation.orgcenter.api.IOrgApi;
import com.ejianc.framework.core.response.CommonResponse;
import org.springframework.beans.factory.annotation.Autowired;
import com.ejianc.framework.core.context.InvocationInfoProxy;
import com.ejianc.framework.core.kit.mapper.BeanMapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.ejianc.framework.core.response.Parameter;
import com.ejianc.framework.core.response.QueryParam;



/**
 * 设计概算变更
 *
 * @author generator
 *
 */
@Controller
@RequestMapping("changedesignestimate")
public class ChangedesignestimateController implements Serializable {
	private static final long serialVersionUID = 1L;

    private Logger logger = LoggerFactory.getLogger(this.getClass());

    @Autowired
    private IBillCodeApi billCodeApi;
    @Autowired
    private IOrgApi iOrgApi;

    private static final String BILL_CODE = "change-budgetmanage-designestimate";//此处需要根据实际修改

    @Autowired
    private IChangedesignestimateService service;

    @Autowired
    private IDesignestimateService  iDesignestimateService;
    /**
     * 初始化更详情页面
     *
     * @param id
     * @return
     */
    @GetMapping(value = "queryDetailByDesignestimateId")
    @ResponseBody
    public CommonResponse<ChangedesignestimateVO> queryDetailByDesignestimateId(@RequestParam(value = "designestimateId") Long designestimateId) {
        DesignestimateEntity designestimateEntity = iDesignestimateService.selectById(designestimateId);
        ChangedesignestimateVO changeVO = BeanMapper.map(designestimateEntity, ChangedesignestimateVO.class);
        Long id = IdWorker.getId();
        changeVO.setDesignestimateId(designestimateId);
        changeVO.setBillState(null);
        changeVO.setCreateTime(null);
        changeVO.setCreateUserCode(null);
        changeVO.setTenantId(null);
        changeVO.setUpdateTime(null);
        changeVO.setUpdateUserCode(null);
        changeVO.setId(id);

        //设计概算清单
        List<DesignestimatedetailEntity> designestimateEntityList = designestimateEntity.getDesignestimatedetailEntities();
        if(designestimateEntityList!=null&&designestimateEntityList.size()>0){
            List<ChangedesignestimatedetailVO> changedesignestimatedetailVOS = new ArrayList<>();
            for(DesignestimatedetailEntity entity : designestimateEntityList){
                ChangedesignestimatedetailVO vo = BeanMapper.map(entity, ChangedesignestimatedetailVO.class);
                vo.setChangeMid(designestimateId);
                vo.setOriginalId(entity.getId());
                vo.setCreateTime(null);
                vo.setCreateUserCode(null);
                vo.setTenantId(null);
                vo.setUpdateTime(null);
                vo.setUpdateUserCode(null);
                vo.setRowState("edit");
                vo.setId(IdWorker.getId());
                changedesignestimatedetailVOS.add(vo);
            }

            //按子目编码排序
            Collections.sort(changedesignestimatedetailVOS, new Comparator<ChangedesignestimatedetailVO>() {
                @Override
                public int compare(ChangedesignestimatedetailVO o1, ChangedesignestimatedetailVO o2) {
                    return o1.getSonItemCode().compareTo(o2.getSonItemCode());
                }
            });

            //构建树形结构
            List<ChangedesignestimatedetailVO> rescChangedesignestimatedetailVOS = createTreeData(changedesignestimatedetailVOS);
            changeVO.setDesignestimatedetailEntities(rescChangedesignestimatedetailVOS);
        }

        return CommonResponse.success("查询详情成功！", changeVO);

    }

    /**
     * @Description saveOrUpdate 新增或者修改
     */
    @RequestMapping(value = "/saveChange", method = RequestMethod.POST)
    @ResponseBody
    public CommonResponse<ChangedesignestimateVO> saveChange(@RequestBody ChangedesignestimateVO saveOrUpdateVO) {
        return service.saveChange(saveOrUpdateVO);
    }

    /**
     * @Description saveOrUpdate 新增或者修改
     */
    @RequestMapping(value = "/saveOrUpdate", method = RequestMethod.POST)
    @ResponseBody
    public CommonResponse<ChangedesignestimateVO> saveOrUpdate(@RequestBody ChangedesignestimateVO saveorUpdateVO) {
    	ChangedesignestimateEntity entity = BeanMapper.map(saveorUpdateVO, ChangedesignestimateEntity.class);
    	if(entity.getId() == null || entity.getId() == 0){
            CommonResponse<String> billCode = billCodeApi.getCodeBatchByRuleCode(BILL_CODE, InvocationInfoProxy.getTenantid());
            if(billCode.isSuccess()) {
//                entity.setCode(billCode.getData());//此处需要根据实际修改 删除本行或者下一行
                entity.setBillCode(billCode.getData());//此处需要根据实际修改 删除本行或者上一行
            }else{
                throw new BusinessException("网络异常， 编码生成失败， 请稍后再试");
            }
        }
    	service.saveOrUpdate(entity, false);
    	ChangedesignestimateVO vo = BeanMapper.map(entity, ChangedesignestimateVO.class);
    	return CommonResponse.success("保存或修改单据成功！",vo);
    }

    /**
     * @Description queryDetail 查询详情
     * @param id
     */
    @RequestMapping(value = "/queryDetail", method = RequestMethod.GET)
    @ResponseBody
    public CommonResponse<ChangedesignestimateVO> queryDetail(Long id) {
    	ChangedesignestimateEntity entity = service.selectById(id);
    	ChangedesignestimateVO vo = BeanMapper.map(entity, ChangedesignestimateVO.class);

        //构建树形结构
        List<ChangedesignestimatedetailVO> quotadesigndetailVO = vo.getDesignestimatedetailEntities();
        //按子目编码排序
        Collections.sort(quotadesigndetailVO, new Comparator<ChangedesignestimatedetailVO>() {
            @Override
            public int compare(ChangedesignestimatedetailVO o1, ChangedesignestimatedetailVO o2) {
                return o1.getSonItemCode().compareTo(o2.getSonItemCode());
            }
        });
        List<ChangedesignestimatedetailVO> resQuotadesigndetailVO = createTreeData(quotadesigndetailVO);
        vo.setDesignestimatedetailEntities(resQuotadesigndetailVO);
        return CommonResponse.success("查询详情数据成功！",vo);
    }

    /**
     * @Description delete 批量删除单据
     * @Param [ids]
     */
    @RequestMapping(value = "/delete", method = RequestMethod.POST)
    @ResponseBody
    public CommonResponse<String> delete(@RequestBody List<ChangedesignestimateVO> vos) {
        return service.delete(vos);
    }

    /**
     * @Description queryList 查询列表
     * @param param
     * @Return com.ejianc.framework.core.response.CommonResponse<java.lang.String>
     */
    @RequestMapping(value = "/queryList", method = RequestMethod.POST)
    @ResponseBody
    public CommonResponse<IPage<ChangedesignestimateVO>> queryList(@RequestBody QueryParam param) {
        /** 模糊搜索配置字段示例 */
        List<String> fuzzyFields = param.getFuzzyFields();
        /** 租户隔离 */
        //param.getParams().put("tenantId", new Parameter(QueryParam.EQ, InvocationInfoProxy.getTenantid()));

        /** 数据隔离 本下 没有组织orgId的删除下面代码 */
//        param.getParams().put("orgId",new Parameter(QueryParam.IN,iOrgApi.findChildrenByParentId(InvocationInfoProxy.getOrgId()).getData().stream().map(OrgVO::getId).collect(Collectors.toList())));

        IPage<ChangedesignestimateEntity> page = service.queryPage(param,false);
        IPage<ChangedesignestimateVO> pageData = new Page<>(page.getCurrent(), page.getSize(), page.getTotal());
  		pageData.setRecords(BeanMapper.mapList(page.getRecords(), ChangedesignestimateVO.class));

        return CommonResponse.success("查询列表数据成功！",pageData);
    }


    /**
     * @Description 导出
     * @param param
     * @Return void
     */
    @RequestMapping(value = "/excelExport", method = RequestMethod.POST)
    @ResponseBody
    public void excelExport(@RequestBody QueryParam param, HttpServletResponse response) {
        /** 模糊搜索配置字段示例 */
        List<String> fuzzyFields = param.getFuzzyFields();
        param.getParams().put("tenant_id",new Parameter(QueryParam.EQ,InvocationInfoProxy.getTenantid()));
        param.setPageIndex(1);
        param.setPageSize(-1);
        /** 数据隔离 本下 没有组织orgId的删除下面代码 */
//        param.getParams().put("orgId",new Parameter(QueryParam.IN,iOrgApi.findChildrenByParentId(InvocationInfoProxy.getOrgId()).getData().stream().map(OrgVO::getId).collect(Collectors.toList())));
        List<ChangedesignestimateEntity> list = service.queryList(param);
        //todo:字段翻译等等
        Map<String, Object> beans = new HashMap<>();
        beans.put("records", list);
        ExcelExport.getInstance().export("Changedesignestimate-export.xlsx", beans, response);
    }

    /**
     * @Description 参照
     * @param
     * @Return void
     */
    @RequestMapping(value = "/refChangedesignestimateData", method = RequestMethod.GET)
    @ResponseBody
    public CommonResponse<IPage<ChangedesignestimateVO>> refChangedesignestimateData(@RequestParam Integer pageNumber, @RequestParam Integer pageSize,
                                                                        String condition,
                                                                        String searchObject,
                                                                        String searchText) {
        QueryParam param = new QueryParam();
        param.setPageSize(pageSize);
        param.setPageIndex(pageNumber);
        param.setSearchText(searchText);
        param.setSearchObject(searchObject);
        /** 租户隔离 */
        //param.getParams().put("tenantId", new Parameter(QueryParam.EQ, InvocationInfoProxy.getTenantid()));
        if(StringUtils.isNotEmpty(condition)){
            /** 处理condition */
            JSONObject _con = JSONObject.parseObject(condition);
        }

        IPage<ChangedesignestimateEntity> page = service.queryPage(param,false);
        IPage<ChangedesignestimateVO> pageData = new Page<>(page.getCurrent(), page.getSize(), page.getTotal());
        pageData.setRecords(BeanMapper.mapList(page.getRecords(), ChangedesignestimateVO.class));

        return CommonResponse.success("查询参照数据成功！",pageData);
     }

    public static List<ChangedesignestimatedetailVO> createTreeData(List<ChangedesignestimatedetailVO> list) {
        List<ChangedesignestimatedetailVO> resp = new ArrayList<>();
        List<String> rootItems = new ArrayList<String>();

        //循环list，放入listMap重
        Map<String, ChangedesignestimatedetailVO> listMap = new HashMap<>();
        for(ChangedesignestimatedetailVO item:list) {
            listMap.put(item.getTid().toString(), item);
        }

        for(int i =0; i<list.size(); i++) {
            ChangedesignestimatedetailVO item = list.get(i);
            String parentId = (item.getTpid() != null) ? item.getTpid().toString():"";
            ChangedesignestimatedetailVO parent = listMap.get(parentId);
            if(parent != null) {
                List<ChangedesignestimatedetailVO> child = (List<ChangedesignestimatedetailVO>) parent.getChildren();
                if(child != null) {
                    child.add(item);
                }else{
                    List<ChangedesignestimatedetailVO> children = new ArrayList<ChangedesignestimatedetailVO>();
                    children.add(item);
                    parent.setChildren(children);
                }
            } else {
                rootItems.add(item.getTid());
            }
        }

        for(String rootId : rootItems) {
            resp.add(listMap.get(rootId));
        }

        return resp;
    }

    /**
     * 页面新增后重新构建树
     *
     * @param
     * @return
     */
    @RequestMapping(value = "/refreshData", method = RequestMethod.POST)
    @ResponseBody
    public CommonResponse<JSONObject> refreshData(@RequestBody List<ChangedesignestimatedetailVO> list, HttpServletResponse response) {
        JSONObject json = new JSONObject();
        Map<String,ChangedesignestimatedetailVO> map = new HashMap<>();
        ChangedesignestimatedetailVO add = list.get(list.size() - 1);
        add.setId(IdWorker.getId());
        add.setTid(String.valueOf(IdWorker.getId()));
        add.setRowState("edit");
        add.setInnercode(add.getSonItemCode());
        if("01".equals(add.getSonItemCode())&&add.getSonItemName().contains("装饰")){
            add.setInnercode("a1");
        }
        //判断是否是装饰工程的子节点编码
        if(add.getSonItemCode().length()>=4){
            //0111、0112、0113、0114、0115这五个节点都是装饰工程下的
            String code = add.getSonItemCode().substring(0,4);
            if("0111".equals(code)||"0112".equals(code)||"0113".equals(code)||"0114".equals(code)||"0115".equals(code)){
//	    		若是的话，将内码改为a1xx
                add.setInnercode("a1"+add.getSonItemCode().substring(2,add.getSonItemCode().length()));
            }
        }
        for(ChangedesignestimatedetailVO vo : list){
            map.put(vo.getInnercode(), vo);
            if(!add.getId().equals(vo.getId()) && add.getInnercode().equals(vo.getInnercode())){
                throw new BusinessException("子目编码已存在，请重新填写");
            }
        }
        //设置父级id
        for(ChangedesignestimatedetailVO vo : list){
            String innercode = vo.getInnercode();
            Long pid = null;
            if(innercode.length()==2){
                //最上级
            }else if(innercode.length()==4){
                String pcode = innercode.substring(0, 2);
                //有父节点
                pid = this.setPid(pcode, map);

            }else if(innercode.length()==6){
                String pcode = innercode.substring(0, 4);
                //有父节点
                pid = this.setPid(pcode, map);
            }else if(innercode.length()>6){
                String pcode = innercode.substring(0, 6);
                //有父节点
                pid = this.setPid(pcode, map);
            }

            if(pid != null){
                vo.setTpid(pid.toString());
            }else {
                vo.setTpid("");
            }
        }

        //清空父级部分数据
        list = clearPData(add, list);

        //按子目编码排序
        Collections.sort(list, new Comparator<ChangedesignestimatedetailVO>() {
            @Override
            public int compare(ChangedesignestimatedetailVO o1, ChangedesignestimatedetailVO o2) {
                return o1.getSonItemCode().compareTo(o2.getSonItemCode());
            }
        });
        //构建树
        List<ChangedesignestimatedetailVO> result = createTreeData(list);

        json.put("dataSource", result);
        return CommonResponse.success(json);
    }

    private List<ChangedesignestimatedetailVO> clearPData(ChangedesignestimatedetailVO handleVO, List<ChangedesignestimatedetailVO> list) {
        if(list == null || list.size() == 0){
            return null;
        }

        String pid = handleVO.getTpid();
        if(pid == null || TextUtils.isEmpty(pid)){
            return list;
        }

        for (int i = 0; i < list.size(); i++) {
            ChangedesignestimatedetailVO vo = list.get(i);
            if(pid.equals(vo.getTid())){
                //清空父级单价和工程量
//                vo.setEngineering(null);
//                vo.setLabor(null);
//                vo.setMaterial(null);
//                vo.setMechanical(null);
//                vo.setManagement(null);
//                vo.setProfit(null);
//                vo.setUnitPrice(null);

                if(vo.getTpid() != null && !TextUtils.isEmpty(vo.getTpid())){
                    clearPData(vo, list);
                    break;
                }
                break;
            }
        }

        return list;
    }

    //子目编码规则定死------分别是2位/4位/6位/6为以上
    private Long setPid(String pcode,Map<String,ChangedesignestimatedetailVO> map){
        if(map.get(pcode)!=null){
            return Long.valueOf(map.get(pcode).getTid());
        }else{
            //父级没有找到，则继续往上找
            if(pcode.length()==2){
                //最上级
                return null;
            }else if(pcode.length()==4){
                String ppcode = pcode.substring(0, 2);
                return this.setPid(ppcode, map);
            }else if(pcode.length()==6){
                String ppcode = pcode.substring(0, 4);
                return this.setPid(ppcode, map);
            }else if(pcode.length()>6){
                String ppcode = pcode.substring(0, 6);
                return this.setPid(ppcode, map);
            }else{
                return null;
            }
        }
    }
}
